# -*- coding: UTF-8 -*-

# ---------------------------------------------------------
# Un analyseur syntaxique pour le lambda-calcul
# Marc Lorenzi - 27 mai 2018
# ---------------------------------------------------------

from lambda_lexer import lex

# ---------------------------------------------------------
# L'analyseur
# En paramètre : une chaîne de caractères s
# Renvoie le lambda-terme correspondant

def parse(s):
    t, reste = parse0(lex(s))
    if len(reste) > 0:
        raise Exception('Unexpected character at position %d' % reste[0][2])
    return t

# ---------------------------------------------------------
# Les fonctions qui suivent prennent en paramètre
# des listes s de lexèmes (cf lambda_lexer.py)

# Analyse d'une expression
# On récupère la liste des expressions irréductibles qui la
# composent, puis on combine ces expressions
def parse0(s):
    ts, reste = parse_list(s)
    t = assembler(ts)
    return t, reste

# Analyse d'une expression irréductible
def parse_irred(s):
    if s[0][0] == 'number': return (church(s[0][1]), s[1:])
    elif s[0][0] == 'alias': return ([3, s[0][1]], s[1:])
    elif s[0][0] == 'var': return ([0, s[0][1]], s[1:])
    elif s[0][0] == 'lambda':
        x = s[1][1]
        if s[2][0] != 'dot':
            raise Exception('Missing dot at character %d' % s[2][2]) 
        t, reste = parse0(s[3:])
        return ([2, x, t], reste)
    elif s[0][0] =='lpar':
        t, reste = parse0(s[1:])
        if reste[0][0] != 'rpar':
            raise Exception('Missing ) at character %d' % s[0][2]) 
        return (t, reste[1:])
    else:
        raise Exception('Syntax Error at character %d' % s[0][2])

# Analyse d'une liste d'expressions irréductibles
def parse_list(s):
    if len(s) == 0: return ([], '')
    elif s[0][0] in ['lambda', 'lpar', 'var', 'maj', 'number', 'alias']:
        t, reste = parse_irred(s)
        ts, reste1 = parse_list(reste)
        return ([t] + ts, reste1)
    else:
        return ([], s)

# Entiers de Church
def church(n):
    s = [0, 'x']
    for k in range(n):
        s = [1, [0, 'f'], s]
    return [2, 'f', [2, 'x', s]]

# Combiner une liste d'expressions
def assembler(ts):
    n = len(ts)
    if n == 0:
        raise Exception('Empty List of Terms')
    else:
        t = ts[0]
        for k in range(1, n):
            t = [1, t, ts[k]]
        return t
